@@ -120,7 +120,6 @@ __BEGIN_DECLS
 #define kIOHIDElementIsNonLinearKey         "IsNonLinear"
 #define kIOHIDElementHasPreferredStateKey   "HasPreferredState"
 #define kIOHIDElementHasNullStateKey        "HasNullState"
-#define kIOHIDElementVendorSpecificKey      "VendorSpecifc"
 #define kIOHIDElementUnitKey                "Unit"
 #define kIOHIDElementUnitExponentKey        "UnitExponent"
 #define kIOHIDElementNameKey                "Name"
@@ -128,6 +127,12 @@ __BEGIN_DECLS
 #define kIOHIDElementDuplicateIndexKey      "DuplicateIndex"
 #define kIOHIDElementParentCollectionKey    "ParentCollection"
 
+#ifndef __ppc__
+    #define kIOHIDElementVendorSpecificKey      "VendorSpecific"
+#else
+    #define kIOHIDElementVendorSpecificKey      "VendorSpecifc"
+#endif
+
 /*!
     @typedef IOHIDElementCookie
     @abstract Abstract data type used as a unique identifier for an element.
