@@ -39,6 +39,7 @@ __BEGIN_DECLS
 #include <mach/mach_interface.h>
 #include <IOKit/iokitmig.h>
 #include <IOKit/IOMessage.h>
+#include <System/libkern/OSCrossEndian.h>
 __END_DECLS
 
 #define connectCheck() do {	    \
@@ -125,8 +126,8 @@ IOHIDDeviceClass::IOHIDDeviceClass()
     fElementCount 		= 0;
     fElements 			= NULL;
 	
-	fCurrentValuesMappedMemory  = NULL;
-	fCurrentValuesMappedMemorySize = NULL;
+	fCurrentValuesMappedMemory  = 0;
+	fCurrentValuesMappedMemorySize = 0;
 
 	fAsyncPrivateDataRef	= NULL;
 	fNotifyPrivateDataRef   = NULL;
@@ -632,21 +633,16 @@ IOReturn IOHIDDeviceClass::close()
 	commandDeviceClosing((IOCDBDeviceInterface **) &fCDBDevice); 
 #endif
 
-//  IOConnectUnmapMemory does not work, so we cannot call it
-// when the user client finally goes away (when our client closes the service)
-// everything will get cleaned up, but this is still ugly
-#if 0
     // finished with the shared memory
-    if (fCurrentValuesMappedMemory != 0)
+    if (fCurrentValuesMappedMemory)
     {
         (void) IOConnectUnmapMemory (	fConnection, 
                                         IOHIDLibUserClientElementValuesType, 
                                         mach_task_self(), 
                                         fCurrentValuesMappedMemory);
-        fCurrentValuesMappedMemory = nil;
-        fCurrentValuesMappedMemorySize = 0;
+        fCurrentValuesMappedMemory      = 0;
+        fCurrentValuesMappedMemorySize  = 0;
     }
-#endif
 
     mach_msg_type_number_t len = 0;
     // kIOCDBUserClientClose,	kIOUCScalarIScalarO,	 0,  0
@@ -750,17 +746,24 @@ IOReturn IOHIDDeviceClass::setElementValue(
         IOHIDElementValue * elementValue = (IOHIDElementValue *)
                 (fCurrentValuesMappedMemory + element.valueLocation);
                 
+        UInt32 totalSize = elementValue->totalSize;
+        ROSETTA_ONLY(
+            totalSize = OSSwapInt32(totalSize);
+        );
         // if size is just one 32bit word
-        if (elementValue->totalSize == sizeof (IOHIDElementValue))
+        if (totalSize == sizeof (IOHIDElementValue))
         {
             //elementValue->cookie = valueEvent->elementCookie;
             elementValue->value[0] = valueEvent->value;
+            ROSETTA_ONLY(
+                elementValue->value[0] = OSSwapInt32(valueEvent->value);
+            );
             //elementValue->timestamp = valueEvent->timestamp;
         }
         // handle the long value size case.
         // we are assuming here that the end user has an allocated
         // longValue buffer.
-        else if (elementValue->totalSize > sizeof (IOHIDElementValue))
+        else if (totalSize > sizeof (IOHIDElementValue))
         {
             UInt32 longValueSize = valueEvent->longValueSize;
             
@@ -768,13 +771,12 @@ IOReturn IOHIDDeviceClass::setElementValue(
                 ( valueEvent->longValue == NULL))
                 return kr;
                 
-            bzero(&(elementValue->value), 
-                (elementValue->totalSize - sizeof(IOHIDElementValue)) + sizeof(UInt32));
+            bzero(&(elementValue->value), element.bytes);
             
             // *** FIX ME ***
             // Since we are setting mapped memory, we should probably
             // hold a shared lock
-            convertByteToWord ((const UInt8 *)valueEvent->longValue, elementValue->value, longValueSize<<3);
+            convertByteToWord ((const UInt8 *)valueEvent->longValue, elementValue->value, longValueSize);
             //elementValue->timestamp = valueEvent->timestamp;
         }
         
@@ -785,8 +787,11 @@ IOReturn IOHIDDeviceClass::setElementValue(
                         
         UInt32			input[1];
         IOByteCount		outputCount = 0;
-                
-        input[0] = (UInt32) elementCookie;
+        
+        input[0] = (UInt32)elementCookie;
+        ROSETTA_ONLY(
+            input[0] = OSSwapInt32((UInt32)elementCookie);
+        );
         
         //  kIOHIDLibUserClientPostElementValue,  kIOUCStructIStructO,    1,	0
         kr = io_connect_method_structureI_structureO(
@@ -847,6 +852,7 @@ IOReturn IOHIDDeviceClass::fillElementValue(IOHIDElementCookie		elementCookie,
     SInt32		value = 0;
     void *		longValue = 0;
     UInt32		longValueSize = 0;
+    UInt32      totalSize;
     UInt64		timestamp = 0;
         
     // get ptr to shared memory for this element
@@ -855,27 +861,39 @@ IOReturn IOHIDDeviceClass::fillElementValue(IOHIDElementCookie		elementCookie,
         IOHIDElementValue * elementValue = (IOHIDElementValue *)
                 (fCurrentValuesMappedMemory + element.valueLocation);
         
+        totalSize = elementValue->totalSize;
+        ROSETTA_ONLY(
+            totalSize = OSSwapInt32(totalSize);
+        );
+
         // if size is just one 32bit word
-        if (elementValue->totalSize == sizeof (IOHIDElementValue))
-        {
-            value = elementValue->value[0];
-            timestamp = *(UInt64 *)& elementValue->timestamp;
-        }
-        // handle the long value size case.
-        // we are assuming here that the end user will deallocate
-        // the longValue buffer.
-        else if (elementValue->totalSize > sizeof (IOHIDElementValue))
-        {
-            longValueSize = element.bytes;
-            longValue = malloc ( longValueSize );
-            bzero(longValue, longValueSize);
+        if ( totalSize >= sizeof (IOHIDElementValue))
+        {        
+            if ( totalSize == sizeof (IOHIDElementValue))
+            {
+                value = elementValue->value[0];
+            }
+            // handle the long value size case.
+            // we are assuming here that the end user will deallocate
+            // the longValue buffer.
+            else
+            {
+                longValueSize = element.bytes;
+                longValue = malloc ( longValueSize );
+                bzero(longValue, longValueSize);
+
+                // *** FIX ME ***
+                // Since we are getting mapped memory, we should probably
+                // hold a shared lock
+                convertWordToByte((const UInt32 *)elementValue->value, (UInt8 *)longValue, longValueSize);
+            }
+            timestamp = *(UInt64 *)&(elementValue->timestamp);
+
+            ROSETTA_ONLY(
+                timestamp 	= OSSwapInt64(timestamp);
+                value 		= OSSwapInt32(value);
+            );
 
-            // *** FIX ME ***
-            // Since we are getting mapped memory, we should probably
-            // hold a shared lock
-            convertWordToByte((const UInt32 *)elementValue->value, (UInt8 *)longValue, longValueSize<<3);
-            
-            timestamp = *(UInt64 *)& elementValue->timestamp;
         }
     }
     
@@ -959,10 +977,17 @@ IOHIDDeviceClass::setReport (	IOHIDReportType			reportType,
         {
             IOHIDReportReq		req;
                         
-            req.reportType = reportType;
-            req.reportID = reportID;
-            req.reportBuffer = reportBuffer;
-            req.reportBufferSize = reportBufferSize;
+            req.reportType          = reportType;
+            req.reportID            = reportID;
+            req.reportBuffer        = reportBuffer;
+            req.reportBufferSize    = reportBufferSize;
+            
+            ROSETTA_ONLY(
+                req.reportType          = OSSwapInt32(req.reportType);
+                req.reportID            = OSSwapInt32(req.reportID);
+                req.reportBuffer        = (void *)OSSwapInt32((uint32_t)req.reportBuffer);
+                req.reportBufferSize    = OSSwapInt32(req.reportBufferSize);
+            );
             
             ret = io_connect_method_structureI_structureO( fConnection, kIOHIDLibUserClientSetReportOOL, (char*)&req, sizeof(req), NULL, &len);
         }    
@@ -1041,12 +1066,23 @@ IOHIDDeviceClass::getReport (	IOHIDReportType			reportType,
             
             len = sizeof(*reportBufferSize);
             
-            req.reportType = reportType;
-            req.reportID = reportID;
-            req.reportBuffer = reportBuffer;
-            req.reportBufferSize = *reportBufferSize;
+            req.reportType          = reportType;
+            req.reportID            = reportID;
+            req.reportBuffer        = reportBuffer;
+            req.reportBufferSize    = *reportBufferSize;
+            
+            ROSETTA_ONLY(
+                req.reportType          = OSSwapInt32(req.reportType);
+                req.reportID            = OSSwapInt32(req.reportID);
+                req.reportBuffer        = (void *)OSSwapInt32((uint32_t)req.reportBuffer);
+                req.reportBufferSize    = OSSwapInt32(req.reportBufferSize);
+            );
             
             ret = io_connect_method_structureI_structureO( fConnection, kIOHIDLibUserClientGetReportOOL, (char*)&req, sizeof(req), (char*)reportBufferSize, &len);
+
+            ROSETTA_ONLY(
+                *reportBufferSize = OSSwapInt32(*reportBufferSize);
+            );
         }    
     }
     
@@ -1127,7 +1163,6 @@ IOHIDDeviceClass::copyMatchingElements(CFDictionaryRef matchingDict, CFArrayRef
                 && CompareProperty(element, matchingDict, CFSTR(kIOHIDElementUnitKey))
                 && CompareProperty(element, matchingDict, CFSTR(kIOHIDElementUnitExponentKey))
                 && CompareProperty(element, matchingDict, CFSTR(kIOHIDElementNameKey))
-                && CompareProperty(element, matchingDict, CFSTR(kIOHIDElementValueLocationKey))
                 && CompareProperty(element, matchingDict, CFSTR(kIOHIDElementDuplicateIndexKey)))
             {            
                 CFArrayAppendValue(tempElements, element);
@@ -1258,7 +1293,11 @@ void IOHIDDeviceClass::_hidReportHandlerCallback(void * target, IOReturn result,
             size = min(size, self->fInputReportBufferSize);
             bzero(self->fInputReportBuffer, size);
 
-            self->convertWordToByte((const UInt32 *)(&(event.value)), (UInt8 *)self->fInputReportBuffer, size << 3);
+            ROSETTA_ONLY(
+                event.value = OSSwapInt32(event.value);
+            );
+
+            self->convertWordToByte((const UInt32 *)(&(event.value)), (UInt8 *)self->fInputReportBuffer, size);
             
         }
         else if (event.longValueSize != 0 && (event.longValue != NULL))
@@ -1297,90 +1336,88 @@ IOHIDDeviceClass::_hidReportCallback(void *refcon, IOReturn result, UInt32 buffe
     free(hidRefcon);
 }
 
+#if __i386__
+    #define ON_INTEL 1
+#else
+    #define ON_INTEL 0
+#endif
+
 //---------------------------------------------------------------------------
 // Not very efficient, will do for now.
 
-#define BIT_MASK(bits)  ((1 << (bits)) - 1)
-
-#define UpdateByteOffsetAndShift(bits, offset, shift)  \
-    do { offset = bits >> 3; shift = bits & 0x07; } while (0)
-
-#define UpdateWordOffsetAndShift(bits, offset, shift)  \
-    do { offset = bits >> 5; shift = bits & 0x1f; } while (0)
-    
-void IOHIDDeviceClass::convertByteToWord( const UInt8 * src,
-                           UInt32 *      dst,
-                           UInt32        bitsToCopy)
+void IOHIDDeviceClass::convertByteToWord( const UInt8 *     src,
+                                           UInt32 *         dst,
+                                           UInt32           bytesToCopy)
 {
-    UInt32 srcOffset;
-    UInt32 srcShift;
-    UInt32 srcStartBit   = 0;
-    UInt32 dstShift      = 0;
-    UInt32 dstStartBit   = 0;
-    UInt32 dstOffset     = 0;
-    UInt32 lastDstOffset = 0;
-    UInt32 word          = 0;
-    UInt8  bitsProcessed;
-    UInt32 totalBitsProcessed = 0;
-
-    while ( bitsToCopy )
+    if ( ON_INTEL || _OSRosettaCheck() )
     {
-        UInt32 tmp;
+        bcopy(src, dst, bytesToCopy);
+    }
+    else 
+    {
+        UInt32 dstOffset    = 0;
+        UInt32 srcOffset    = 0;
+        UInt32 temp         = 0;
+        UInt32 tempShift    = 0;
+        
+        while ( bytesToCopy >= 4 )
+        {                
+            dst[dstOffset] = OSSwapInt32(*((UInt32 *)&(src[srcOffset])));
 
-        UpdateByteOffsetAndShift( srcStartBit, srcOffset, srcShift );
-
-        bitsProcessed = min( bitsToCopy,
-                             min( 8 - srcShift, 32 - dstShift ) );
-
-        tmp = (src[srcOffset] >> srcShift) & BIT_MASK(bitsProcessed);
-
-        word |= ( tmp << dstShift );
-
-        dstStartBit += bitsProcessed;
-        srcStartBit += bitsProcessed;
-        bitsToCopy  -= bitsProcessed;
-		totalBitsProcessed += bitsProcessed;
-
-        UpdateWordOffsetAndShift( dstStartBit, dstOffset, dstShift );
-
-        if ( ( dstOffset != lastDstOffset ) || ( bitsToCopy == 0 ) )
+            srcOffset   += 4;
+            bytesToCopy -= 4;
+            dstOffset   ++;
+        }
+        
+        while ( bytesToCopy )
         {
-            dst[lastDstOffset] = word;
-            word = 0;
-            lastDstOffset = dstOffset;
+            temp |= src[srcOffset++] << tempShift;
+            
+            tempShift += 8;
+            bytesToCopy --;
+            
+            if ( !bytesToCopy )
+                dst[dstOffset] = temp;
         }
     }
 }
 
 void IOHIDDeviceClass::convertWordToByte( const UInt32 * src,
                            UInt8 *        dst,
-                           UInt32         bitsToCopy)
+                           UInt32         bytesToCopy)
 {
-    UInt32 dstOffset;
-    UInt32 dstShift;
-    UInt32 dstStartBit = 0;
-    UInt32 srcShift    = 0;
-    UInt32 srcStartBit = 0;
-    UInt32 srcOffset   = 0;
-    UInt8  bitsProcessed;
-    UInt32 tmp;
-
-    while ( bitsToCopy )
+    if ( ON_INTEL || _OSRosettaCheck() )
     {
-        UpdateByteOffsetAndShift( dstStartBit, dstOffset, dstShift );
+        bcopy(src, dst, bytesToCopy);
+    }
+    else 
+    {
+        UInt32 dstOffset    = 0;
+        UInt32 srcOffset    = 0;
+        UInt32 temp         = 0;
+        UInt32 tmpOffset    = 0;
+        
+        while ( bytesToCopy )
+        {        
+            temp = OSSwapInt32(src[srcOffset++]);
 
-        bitsProcessed = min( bitsToCopy,
-                             min( 8 - dstShift, 32 - srcShift ) );
-
-        tmp = (src[srcOffset] >> srcShift) & BIT_MASK(bitsProcessed);
-
-        dst[dstOffset] |= ( tmp << dstShift );
-
-        dstStartBit += bitsProcessed;
-        srcStartBit += bitsProcessed;
-        bitsToCopy  -= bitsProcessed;
-
-        UpdateWordOffsetAndShift( srcStartBit, srcOffset, srcShift );
+            if ( bytesToCopy >= 4 )
+            {
+                *((UInt32 *)&(dst[dstOffset])) = temp;
+            
+                bytesToCopy -= 4;
+                dstOffset   += 4;
+            }
+            else 
+            {
+                tmpOffset = 0;
+                while ( bytesToCopy )
+                {
+                    dst[dstOffset++] = ((UInt8 *)&temp)[tmpOffset++];
+                    bytesToCopy--;
+                }
+            }
+        }
     }
 }
 
@@ -2023,7 +2060,6 @@ kern_return_t IOHIDDeviceClass::CreateLeafElements (CFDictionaryRef properties,
             } while ( 0 );
 
         }
-        
         CFRelease(dictionary);
     }
     // this case should not happen, something else was found
